@@ -71,7 +71,7 @@ static int amr_write_packet(AVFormatContext *s, AVPacket *pkt)
 }
 #endif /* CONFIG_AMR_MUXER */
 
-static int amr_probe(AVProbeData *p)
+static int amr_probe(const AVProbeData *p)
 {
     // Only check for "#!AMR" which could be amr-wb, amr-nb.
     // This will also trigger multichannel files: "#!AMR_MC1.0\n" and
@@ -90,15 +90,13 @@ static int amr_read_header(AVFormatContext *s)
     AVStream *st;
     uint8_t header[9];
 
-    if (avio_read(pb, header, 6) != 6)
-        return AVERROR_INVALIDDATA;
+    avio_read(pb, header, 6);
 
     st = avformat_new_stream(s, NULL);
     if (!st)
         return AVERROR(ENOMEM);
     if (memcmp(header, AMR_header, 6)) {
-        if (avio_read(pb, header + 6, 3) != 3)
-            return AVERROR_INVALIDDATA;
+        avio_read(pb, header + 6, 3);
         if (memcmp(header, AMRWB_header, 9)) {
             return -1;
         }
@@ -178,7 +176,7 @@ AVInputFormat ff_amr_demuxer = {
 #endif
 
 #if CONFIG_AMRNB_DEMUXER
-static int amrnb_probe(AVProbeData *p)
+static int amrnb_probe(const AVProbeData *p)
 {
     int mode, i = 0, valid = 0, invalid = 0;
     const uint8_t *b = p->buf;
@@ -234,7 +232,7 @@ AVInputFormat ff_amrnb_demuxer = {
 #endif
 
 #if CONFIG_AMRWB_DEMUXER
-static int amrwb_probe(AVProbeData *p)
+static int amrwb_probe(const AVProbeData *p)
 {
     int mode, i = 0, valid = 0, invalid = 0;
     const uint8_t *b = p->buf;
