@@ -23,7 +23,7 @@
 #include "avformat.h"
 #include "riff.h"
 
-static int lvf_probe(AVProbeData *p)
+static int lvf_probe(const AVProbeData *p)
 {
     if (AV_RL32(p->buf) != MKTAG('L', 'V', 'F', 'F'))
         return 0;
@@ -106,7 +106,6 @@ static int lvf_read_packet(AVFormatContext *s, AVPacket *pkt)
     unsigned size, flags, timestamp, id;
     int64_t pos;
     int ret, is_video = 0;
-    int stream_index;
 
     pos = avio_tell(s->pb);
     while (!avio_feof(s->pb)) {
@@ -122,15 +121,12 @@ static int lvf_read_packet(AVFormatContext *s, AVPacket *pkt)
         case MKTAG('0', '1', 'w', 'b'):
             if (size < 8)
                 return AVERROR_INVALIDDATA;
-            stream_index = is_video ? 0 : 1;
-            if (stream_index >= s->nb_streams)
-                return AVERROR_INVALIDDATA;
             timestamp = avio_rl32(s->pb);
             flags = avio_rl32(s->pb);
             ret = av_get_packet(s->pb, pkt, size - 8);
             if (flags & (1 << 12))
                 pkt->flags |= AV_PKT_FLAG_KEY;
-            pkt->stream_index = stream_index;
+            pkt->stream_index = is_video ? 0 : 1;
             pkt->pts          = timestamp;
             pkt->pos          = pos;
             return ret;
