@@ -36,7 +36,7 @@ typedef struct {
 } MicroDVDContext;
 
 
-static int microdvd_probe(AVProbeData *p)
+static int microdvd_probe(const AVProbeData *p)
 {
     unsigned char c;
     const uint8_t *ptr = p->buf;
@@ -81,7 +81,7 @@ static int microdvd_read_header(AVFormatContext *s)
     AVRational pts_info = (AVRational){ 2997, 125 };  /* default: 23.976 fps */
     MicroDVDContext *microdvd = s->priv_data;
     AVStream *st = avformat_new_stream(s, NULL);
-    int i = 0, ret;
+    int i = 0;
     char line_buf[MAX_LINESIZE];
     int has_real_fps = 0;
 
@@ -94,7 +94,6 @@ static int microdvd_read_header(AVFormatContext *s)
         int64_t pos = avio_tell(s->pb);
         int len = ff_get_line(s->pb, line_buf, sizeof(line_buf));
         char *line = line_buf;
-        int64_t pts;
 
         if (!strncmp(line, bom, 3))
             line += 3;
@@ -103,6 +102,8 @@ static int microdvd_read_header(AVFormatContext *s)
         if (!len)
             break;
         line[strcspn(line, "\r\n")] = 0;
+        if (!*p)
+            continue;
         if (i++ < 3) {
             int frame;
             double fps;
@@ -117,10 +118,8 @@ static int microdvd_read_header(AVFormatContext *s)
             }
             if (!st->codecpar->extradata && sscanf(line, "{DEFAULT}{}%c", &c) == 1) {
                 st->codecpar->extradata = av_strdup(line + 11);
-                if (!st->codecpar->extradata) {
-                    ret = AVERROR(ENOMEM);
-                    goto fail;
-                }
+                if (!st->codecpar->extradata)
+                    return AVERROR(ENOMEM);
                 st->codecpar->extradata_size = strlen(st->codecpar->extradata) + 1;
                 continue;
             }
@@ -137,16 +136,11 @@ static int microdvd_read_header(AVFormatContext *s)
         SKIP_FRAME_ID;
         if (!*p)
             continue;
-        pts = get_pts(line);
-        if (pts == AV_NOPTS_VALUE)
-            continue;
         sub = ff_subtitles_queue_insert(&microdvd->q, p, strlen(p), 0);
-        if (!sub) {
-            ret = AVERROR(ENOMEM);
-            goto fail;
-        }
+        if (!sub)
+            return AVERROR(ENOMEM);
         sub->pos = pos;
-        sub->pts = pts;
+        sub->pts = get_pts(line);
         sub->duration = get_duration(line);
     }
     ff_subtitles_queue_finalize(s, &microdvd->q);
@@ -161,9 +155,6 @@ static int microdvd_read_header(AVFormatContext *s)
     st->codecpar->codec_type = AVMEDIA_TYPE_SUBTITLE;
     st->codecpar->codec_id   = AV_CODEC_ID_MICRODVD;
     return 0;
-fail:
-    ff_subtitles_queue_clean(&microdvd->q);
-    return ret;
 }
 
 static int microdvd_read_packet(AVFormatContext *s, AVPacket *pkt)
