@@ -33,7 +33,7 @@ typedef struct {
     FFDemuxSubtitlesQueue q;
 } MPL2Context;
 
-static int mpl2_probe(AVProbeData *p)
+static int mpl2_probe(const AVProbeData *p)
 {
     int i;
     char c;
@@ -55,7 +55,7 @@ static int mpl2_probe(AVProbeData *p)
     return AVPROBE_SCORE_MAX;
 }
 
-static int read_ts(char **line, int64_t *pts_start, int64_t *duration)
+static int read_ts(char **line, int64_t *pts_start, int *duration)
 {
     char c;
     int len;
@@ -69,10 +69,7 @@ static int read_ts(char **line, int64_t *pts_start, int64_t *duration)
     }
     if (sscanf(*line, "[%"SCNd64"][%"SCNd64"]%c%n",
                pts_start, &end, &c, &len) >= 3) {
-        if (end < *pts_start || end - (uint64_t)*pts_start > INT64_MAX) {
-            *duration = -1;
-        } else
-            *duration = end - *pts_start;
+        *duration = end - *pts_start;
         *line += len - 1;
         return 0;
     }
@@ -100,7 +97,7 @@ static int mpl2_read_header(AVFormatContext *s)
         const int64_t pos = avio_tell(s->pb);
         int len = ff_get_line(s->pb, line, sizeof(line));
         int64_t pts_start;
-        int64_t duration;
+        int duration;
 
         if (!len)
             break;
@@ -111,10 +108,8 @@ static int mpl2_read_header(AVFormatContext *s)
             AVPacket *sub;
 
             sub = ff_subtitles_queue_insert(&mpl2->q, p, strlen(p), 0);
-            if (!sub) {
-                ff_subtitles_queue_clean(&mpl2->q);
+            if (!sub)
                 return AVERROR(ENOMEM);
-            }
             sub->pos = pos;
             sub->pts = pts_start;
             sub->duration = duration;
