@@ -107,7 +107,7 @@ static int spdif_get_offset_and_codec(AVFormatContext *s,
    samples = 4096 */
 #define SPDIF_MAX_OFFSET 16384
 
-static int spdif_probe(AVProbeData *p)
+static int spdif_probe(const AVProbeData *p)
 {
     enum AVCodecID codec;
     return ff_spdif_probe (p->buf, p->buf_size, &codec);
@@ -229,7 +229,7 @@ int ff_spdif_read_packet(AVFormatContext *s, AVPacket *pkt)
     if (!s->bit_rate && s->streams[0]->codecpar->sample_rate)
         /* stream bitrate matches 16-bit stereo PCM bitrate for currently
            supported codecs */
-        s->bit_rate = 2 * 16LL * s->streams[0]->codecpar->sample_rate;
+        s->bit_rate = 2 * 16 * s->streams[0]->codecpar->sample_rate;
 
     return 0;
 }
