@@ -392,12 +392,7 @@ static int tls_read(URLContext *h, uint8_t *buf, int len)
     int size, ret;
     int min_enc_buf_size = len + SCHANNEL_FREE_BUFFER_SIZE;
 
-    /* If we have some left-over data from previous network activity,
-     * return it first in case it is enough. It may contain
-     * data that is required to know whether this connection
-     * is still required or not, esp. in case of HTTP keep-alive
-     * connections. */
-    if (c->dec_buf_offset > 0)
+    if (len <= c->dec_buf_offset)
         goto cleanup;
 
     if (c->sspi_close_notify)
@@ -429,7 +424,7 @@ static int tls_read(URLContext *h, uint8_t *buf, int len)
         c->enc_buf_offset += ret;
     }
 
-    while (c->enc_buf_offset > 0 && sspi_ret == SEC_E_OK) {
+    while (c->enc_buf_offset > 0 && sspi_ret == SEC_E_OK && c->dec_buf_offset < len) {
         /*  input buffer */
         init_sec_buffer(&inbuf[0], SECBUFFER_DATA, c->enc_buf, c->enc_buf_offset);
 
